@@ -18,7 +18,7 @@
 %                    Based on kmiya's sixel (2014-03-28)                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,41 +40,39 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/color.h"
-#include "magick/color-private.h"
-#include "magick/colormap.h"
-#include "magick/colormap-private.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/geometry.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/locale_.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/pixel-accessor.h"
-#include "magick/pixel-private.h"
-#include "magick/quantize.h"
-#include "magick/quantum-private.h"
-#include "magick/resize.h"
-#include "magick/resource_.h"
-#include "magick/splay-tree.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/thread-private.h"
-#include "magick/module.h"
-#include "magick/threshold.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/color.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/colormap.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/geometry.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/pixel-private.h"
+#include "MagickCore/quantize.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resize.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/splay-tree.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/thread-private.h"
+#include "MagickCore/module.h"
+#include "MagickCore/threshold.h"
+#include "MagickCore/utility.h"
 
 /*
   Definitions
@@ -85,7 +83,7 @@
 /*
   Macros
 */
-#define SIXEL_RGB(r, g, b) ((int) (((ssize_t) ((r) & 0xff) << 16) + (((g) & 0xff) << 8) +  ((b) & 0xff)))
+#define SIXEL_RGB(r, g, b) ((int) (((ssize_t) (r) << 16) + ((g) << 8) +  (b)))
 #define SIXEL_PALVAL(n,a,m) ((int) (((ssize_t) (n) * (a) + ((m) / 2)) / (m)))
 #define SIXEL_XRGB(r,g,b) SIXEL_RGB(SIXEL_PALVAL(r, 255, 100), SIXEL_PALVAL(g, 255, 100), SIXEL_PALVAL(b, 255, 100))
 
@@ -117,7 +115,7 @@ typedef struct sixel_output {
 
     Image *image;
     int pos;
-    unsigned char buffer[MagickMax(SIXEL_OUTPUT_PACKET_SIZE*2,MagickPathExtent)];
+    unsigned char buffer[1];
 
 } sixel_output_t;
 
@@ -144,7 +142,7 @@ static int const sixel_default_color_table[] = {
   Forward declarations.
 */
 static MagickBooleanType
-  WriteSIXELImage(const ImageInfo *,Image *);
+  WriteSIXELImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 static int hue_to_rgb(int n1, int n2, int hue)
 {
@@ -159,13 +157,13 @@ static int hue_to_rgb(int n1, int n2, int hue)
     }
 
     if (hue < (HLSMAX / 6)) {
-        return (n1 + (((ssize_t) (n2 - n1) * hue + (HLSMAX / 12)) / (HLSMAX / 6)));
+        return (n1 + (((n2 - n1) * hue + (HLSMAX / 12)) / (HLSMAX / 6)));
     }
     if (hue < (HLSMAX / 2)) {
         return (n2);
     }
     if (hue < ((HLSMAX * 2) / 3)) {
-        return (n1 + (((ssize_t) (n2 - n1) * (((HLSMAX * 2) / 3) - hue) + (HLSMAX / 12))/(HLSMAX / 6)));
+        return (n1 + (((n2 - n1) * (((HLSMAX * 2) / 3) - hue) + (HLSMAX / 12))/(HLSMAX / 6)));
     }
     return (n1);
 }
@@ -185,11 +183,11 @@ static int hls_to_rgb(int hue, int lum, int sat)
         } else {
             Magic2 = (int) (lum + sat - (((ssize_t) lum * sat) + (HLSMAX / 2)) / HLSMAX);
         }
-        Magic1 = (int) (2 * (ssize_t) lum) - Magic2;
+        Magic1 = 2 * lum - Magic2;
 
-        B = (hue_to_rgb(Magic1, Magic2, (ssize_t) hue + (HLSMAX / 3)) * (ssize_t) RGBMAX + (HLSMAX / 2)) / HLSMAX;
-        R = (hue_to_rgb(Magic1, Magic2, hue) * RGBMAX + (ssize_t) (HLSMAX / 2)) / HLSMAX;
-        G = (hue_to_rgb(Magic1, Magic2, (ssize_t) hue - (HLSMAX / 3)) * (ssize_t) RGBMAX + (HLSMAX/2)) / HLSMAX;
+        R = (hue_to_rgb(Magic1, Magic2, hue + (HLSMAX / 3)) * RGBMAX + (HLSMAX / 2)) / HLSMAX;
+        G = (hue_to_rgb(Magic1, Magic2, hue) * RGBMAX + (HLSMAX / 2)) / HLSMAX;
+        B = (hue_to_rgb(Magic1, Magic2, hue - (HLSMAX / 3)) * RGBMAX + (HLSMAX/2)) / HLSMAX;
     }
     return SIXEL_RGB(R, G, B);
 }
@@ -235,7 +233,8 @@ MagickBooleanType sixel_decode(Image *image,
                                size_t                     /* out */ *pwidth,    /* image width */
                                size_t                     /* out */ *pheight,   /* image height */
                                unsigned char              /* out */ **palette,  /* ARGB palette */
-                               size_t                     /* out */ *ncolors    /* palette size (<= 256) */)
+                               size_t                     /* out */ *ncolors,    /* palette size (<= 256) */
+  ExceptionInfo *exception)
 {
     int n, i, r, g, b, sixel_vertical_mask, c;
     int posision_x, posision_y;
@@ -263,7 +262,7 @@ MagickBooleanType sixel_decode(Image *image,
 
     imsx = 2048;
     imsy = 2048;
-    if (SetImageExtent(image,imsx,imsy) == MagickFalse)
+    if (SetImageExtent(image,imsx,imsy,exception) == MagickFalse)
       return(MagickFalse);
     imbuf = (unsigned char *) AcquireQuantumMemory(imsx , imsy);
 
@@ -342,8 +341,8 @@ MagickBooleanType sixel_decode(Image *image,
                     if (param[2] == 0) {
                         param[2] = 10;
                     }
-                    attributed_pan = (int) (((size_t) attributed_pan * param[2]) / 10);
-                    attributed_pad = (int) (((size_t) attributed_pad * param[2]) / 10);
+                    attributed_pan = (int) (((ssize_t) attributed_pan * param[2]) / 10);
+                    attributed_pad = (int) (((ssize_t) attributed_pad * param[2]) / 10);
                     if (attributed_pan <= 0) attributed_pan = 1;
                     if (attributed_pad <= 0) attributed_pad = 1;
                 }
@@ -357,8 +356,8 @@ MagickBooleanType sixel_decode(Image *image,
 
             if (n > 0) attributed_pad = param[0];
             if (n > 1) attributed_pan = param[1];
-            if (n > 2 && param[2] > 0) attributed_ph = param[2] & 0xffff;
-            if (n > 3 && param[3] > 0) attributed_pv = param[3] & 0xffff;
+            if (n > 2 && param[2] > 0) attributed_ph = param[2];
+            if (n > 3 && param[3] > 0) attributed_pv = param[3];
 
             if (attributed_pan <= 0) attributed_pan = 1;
             if (attributed_pad <= 0) attributed_pad = 1;
@@ -366,7 +365,7 @@ MagickBooleanType sixel_decode(Image *image,
             if (imsx < attributed_ph || imsy < attributed_pv) {
                 dmsx = imsx > attributed_ph ? imsx : attributed_ph;
                 dmsy = imsy > attributed_pv ? imsy : attributed_pv;
-                if (SetImageExtent(image,dmsx,dmsy) == MagickFalse)
+                if (SetImageExtent(image,dmsx,dmsy,exception) == MagickFalse)
                   break;
                 dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx , dmsy);
                 if (dmbuf == (unsigned char *) NULL) {
@@ -375,7 +374,7 @@ MagickBooleanType sixel_decode(Image *image,
                 }
                 (void) memset(dmbuf, background_color_index, (size_t) dmsx * dmsy);
                 for (y = 0; y < imsy; ++y) {
-                    (void) memcpy(dmbuf + dmsx * y, imbuf + (size_t) imsx * y, imsx);
+                    (void) memcpy(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
                 }
                 imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
                 imsx = dmsx;
@@ -444,7 +443,7 @@ MagickBooleanType sixel_decode(Image *image,
 
                 dmsx = nx;
                 dmsy = ny;
-                if (SetImageExtent(image,dmsx,dmsy) == MagickFalse)
+                if (SetImageExtent(image,dmsx,dmsy,exception) == MagickFalse)
                   break;
                 dmbuf = (unsigned char *) AcquireQuantumMemory(dmsx , dmsy);
                 if (dmbuf == (unsigned char *) NULL) {
@@ -453,7 +452,7 @@ MagickBooleanType sixel_decode(Image *image,
                 }
                 (void) memset(dmbuf, background_color_index, (size_t) dmsx * dmsy);
                 for (y = 0; y < imsy; ++y) {
-                    (void) memcpy(dmbuf + dmsx * y, imbuf + (size_t) imsx * y, imsx);
+                    (void) memcpy(dmbuf + dmsx * y, imbuf + imsx * y, imsx);
                 }
                 imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
                 imsx = dmsx;
@@ -541,7 +540,7 @@ MagickBooleanType sixel_decode(Image *image,
     if (imsx > max_x || imsy > max_y) {
         dmsx = max_x;
         dmsy = max_y;
-        if (SetImageExtent(image,dmsx,dmsy) == MagickFalse)
+        if (SetImageExtent(image,dmsx,dmsy,exception) == MagickFalse)
           {
             imbuf = (unsigned char *) RelinquishMagickMemory(imbuf);
             return (MagickFalse);
@@ -579,7 +578,7 @@ sixel_output_t *sixel_output_create(Image *image)
 {
     sixel_output_t *output;
 
-    output = (sixel_output_t *) AcquireMagickMemory(sizeof(sixel_output_t));
+    output = (sixel_output_t *) AcquireQuantumMemory(sizeof(sixel_output_t) + SIXEL_OUTPUT_PACKET_SIZE * 2, 1);
     if (output == (sixel_output_t *) NULL)
       return((sixel_output_t *) NULL);
     output->has_8bit_control = 0;
@@ -622,9 +621,7 @@ static int sixel_put_flash(sixel_output_t *const context)
 
     if (context->save_count > 3) {
         /* DECGRI Graphics Repeat Introducer ! Pn Ch */
-        nwrite=FormatLocaleString((char *) context->buffer+context->pos,
-          sizeof(context->buffer),"!%d%c",context->save_count,
-          context->save_pixel);
+        nwrite = sprintf((char *)context->buffer + context->pos, "!%d%c", context->save_count, context->save_pixel);
         if (nwrite <= 0) {
             return (-1);
         }
@@ -689,8 +686,8 @@ static int sixel_put_node(sixel_output_t *const context, int x,
     if (ncolors != 2 || keycolor == -1) {
         /* designate palette index */
         if (context->active_palette != np->color) {
-            nwrite=FormatLocaleString((char *) context->buffer+context->pos,
-              sizeof(context->buffer),"#%d",np->color);
+            nwrite = sprintf((char *)context->buffer + context->pos,
+                             "#%d", np->color);
             sixel_advance(context, nwrite);
             context->active_palette = np->color;
         }
@@ -723,10 +720,10 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
     int x, y, i, n, c;
     int left, right;
     int pix;
-    size_t len;
     unsigned char *map;
     sixel_node_t *np, *tp, top;
     int nwrite;
+    size_t len;
 
     context->pos = 0;
 
@@ -742,18 +739,15 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
     (void) memset(map, 0, len);
 
     if (context->has_8bit_control) {
-        nwrite=FormatLocaleString((char *) context->buffer,
-          sizeof(context->buffer),"\x90" "0;0;0" "q");
+        nwrite = sprintf((char *)context->buffer, "\x90" "0;0;0" "q");
     } else {
-        nwrite=FormatLocaleString((char *) context->buffer,
-          sizeof(context->buffer),"\x1bP" "0;0;0" "q");
+        nwrite = sprintf((char *)context->buffer, "\x1bP" "0;0;0" "q");
     }
     if (nwrite <= 0) {
         return (MagickFalse);
     }
     sixel_advance(context, nwrite);
-    nwrite=FormatLocaleString((char *) context->buffer+context->pos,
-      sizeof(context->buffer),"\"1;1;%d;%d",(int) width,(int) height);
+    nwrite = sprintf((char *)context->buffer + context->pos, "\"1;1;%d;%d", (int) width, (int) height);
     if (nwrite <= 0) {
         RelinquishNodesAndMap;
         return (MagickFalse);
@@ -763,11 +757,11 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
     if (ncolors != 2 || keycolor == -1) {
         for (n = 0; n < (ssize_t) ncolors; n++) {
             /* DECGCI Graphics Color Introducer  # Pc ; Pu; Px; Py; Pz */
-            nwrite=FormatLocaleString((char *) context->buffer+context->pos,
-              sizeof(context->buffer),"#%d;2;%d;%d;%d",n,
-              (palette[n * 3 + 0] * 100 + 127) / 255,
-              (palette[n * 3 + 1] * 100 + 127) / 255,
-              (palette[n * 3 + 2] * 100 + 127) / 255);
+            nwrite = sprintf((char *)context->buffer + context->pos, "#%d;2;%d;%d;%d",
+                             n,
+                             (palette[n * 3 + 0] * 100 + 127) / 255,
+                             (palette[n * 3 + 1] * 100 + 127) / 255,
+                             (palette[n * 3 + 2] * 100 + 127) / 255);
             if (nwrite <= 0) {
                 RelinquishNodesAndMap;
                 return (MagickFalse);
@@ -900,7 +894,7 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
 
     /* flush buffer */
     if (context->pos > 0) {
-        (void) WriteBlob(context->image,context->pos,context->buffer);
+        WriteBlob(context->image,context->pos,context->buffer);
     }
 
     RelinquishNodesAndMap;
@@ -924,7 +918,8 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
 %
 %  The format of the IsSIXEL method is:
 %
-%      MagickBooleanType IsSIXEL(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsSIXEL(const unsigned char *magick,
+%        const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -934,7 +929,8 @@ static MagickBooleanType sixel_encode_impl(unsigned char *pixels, size_t width,s
 %    o length: Specifies the length of the magick string.
 %
 */
-static MagickBooleanType IsSIXEL(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsSIXEL(const unsigned char *magick,
+  const size_t length)
 {
   const unsigned char
     *end = magick + length;
@@ -970,7 +966,8 @@ static MagickBooleanType IsSIXEL(const unsigned char *magick,const size_t length
 %
 %  The format of the ReadSIXELImage method is:
 %
-%      Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exception)
+%      Image *ReadSIXELImage(const ImageInfo *image_info,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -990,17 +987,14 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   MagickBooleanType
     status;
 
-  char
+  register char
     *p;
 
-  IndexPacket
-    *indexes;
-
-  ssize_t
+  register ssize_t
     x;
 
-  PixelPacket
-    *r;
+  register Quantum
+    *q;
 
   size_t
     length;
@@ -1019,12 +1013,12 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickCoreSignature);
-  if (IsEventLogging() != MagickFalse)
+  if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  image=AcquireImage(image_info);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -1034,41 +1028,35 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   /*
     Read SIXEL file.
   */
-  length=MaxTextExtent;
-  sixel_buffer=(char *) AcquireQuantumMemory((size_t) length+MaxTextExtent,
+  length=MagickPathExtent;
+  sixel_buffer=(char *) AcquireQuantumMemory((size_t) length+MagickPathExtent,
     sizeof(*sixel_buffer));
   p=sixel_buffer;
   if (sixel_buffer != (char *) NULL)
     while (ReadBlobString(image,p) != (char *) NULL)
     {
-      ssize_t
-        offset;
-
       if ((*p == '#') && ((p == sixel_buffer) || (*(p-1) == '\n')))
         continue;
       if ((*p == '}') && (*(p+1) == ';'))
         break;
-      p+=(ptrdiff_t) strlen(p);
-      offset=p-sixel_buffer;
-      if ((size_t) (offset+MaxTextExtent+1) < length)
+      p+=strlen(p);
+      if ((size_t) (p-sixel_buffer+MagickPathExtent+1) < length)
         continue;
       length<<=1;
       sixel_buffer=(char *) ResizeQuantumMemory(sixel_buffer,length+
-        MaxTextExtent+1,sizeof(*sixel_buffer));
+        MagickPathExtent+1,sizeof(*sixel_buffer));
       if (sixel_buffer == (char *) NULL)
         break;
-      p=sixel_buffer+offset;
+      p=sixel_buffer+strlen(sixel_buffer);
     }
   if (sixel_buffer == (char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   sixel_buffer[length]='\0';
   /*
-    Decode SIXEL.
+    Decode SIXEL
   */
   sixel_pixels=(unsigned char *) NULL;
-  status=sixel_decode(image,(unsigned char *) sixel_buffer,&sixel_pixels,
-    &image->columns,&image->rows,&sixel_palette,&image->colors);
-  if (status == MagickFalse)
+  if (sixel_decode(image,(unsigned char *) sixel_buffer,&sixel_pixels,&image->columns,&image->rows,&sixel_palette,&image->colors,exception) == MagickFalse)
     {
       sixel_buffer=(char *) RelinquishMagickMemory(sixel_buffer);
       if (sixel_pixels != (unsigned char *) NULL)
@@ -1078,26 +1066,26 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   sixel_buffer=(char *) RelinquishMagickMemory(sixel_buffer);
   image->depth=24;
   image->storage_class=PseudoClass;
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     {
       sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
       sixel_palette=(unsigned char *) RelinquishMagickMemory(sixel_palette);
-      InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
-  if (AcquireImageColormap(image,image->colors) == MagickFalse)
+
+  if (AcquireImageColormap(image,image->colors, exception) == MagickFalse)
     {
       sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
       sixel_palette=(unsigned char *) RelinquishMagickMemory(sixel_palette);
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
-  for (i = 0; i < (ssize_t) image->colors; ++i)
-  {
-    image->colormap[i].red=ScaleCharToQuantum(sixel_palette[i * 4 + 0]);
-    image->colormap[i].green=ScaleCharToQuantum(sixel_palette[i * 4 + 1]);
-    image->colormap[i].blue=ScaleCharToQuantum(sixel_palette[i * 4 + 2]);
+  for (i = 0; i < (ssize_t) image->colors; ++i) {
+    image->colormap[i].red   = ScaleCharToQuantum(sixel_palette[i * 4 + 0]);
+    image->colormap[i].green = ScaleCharToQuantum(sixel_palette[i * 4 + 1]);
+    image->colormap[i].blue  = ScaleCharToQuantum(sixel_palette[i * 4 + 2]);
   }
+
   j=0;
   if (image_info->ping == MagickFalse)
     {
@@ -1106,17 +1094,14 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
       */
       for (y=0; y < (ssize_t) image->rows; y++)
       {
-        r=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
-        if (r == (PixelPacket *) NULL)
+        q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
+        if (q == (Quantum *) NULL)
           break;
-        indexes=GetAuthenticIndexQueue(image);
         for (x=0; x < (ssize_t) image->columns; x++)
         {
           j=(ssize_t) sixel_pixels[y * image->columns + x];
-          j=ConstrainColormapIndex(image,j);
-          SetPixelIndex(indexes+x,j);
-          SetPixelRGBO(r,image->colormap+(ssize_t) j);
-          r++;
+          SetPixelIndex(image,j,q);
+          q+=GetPixelChannels(image);
         }
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
@@ -1133,10 +1118,7 @@ static Image *ReadSIXELImage(const ImageInfo *image_info,ExceptionInfo *exceptio
   */
   sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
   sixel_palette=(unsigned char *) RelinquishMagickMemory(sixel_palette);
-  if (CloseBlob(image) == MagickFalse)
-    status=MagickFalse;
-  if (status == MagickFalse)
-    return(DestroyImageList(image));
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 
@@ -1168,21 +1150,17 @@ ModuleExport size_t RegisterSIXELImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("SIXEL");
+  entry=AcquireMagickInfo("SIXEL","SIXEL","DEC SIXEL Graphics Format");
   entry->decoder=(DecodeImageHandler *) ReadSIXELImage;
   entry->encoder=(EncodeImageHandler *) WriteSIXELImage;
   entry->magick=(IsImageFormatHandler *) IsSIXEL;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("DEC SIXEL Graphics Format");
-  entry->magick_module=ConstantString("SIXEL");
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("SIX");
+  entry=AcquireMagickInfo("SIXEL","SIX","DEC SIXEL Graphics Format");
   entry->decoder=(DecodeImageHandler *) ReadSIXELImage;
   entry->encoder=(EncodeImageHandler *) WriteSIXELImage;
   entry->magick=(IsImageFormatHandler *) IsSIXEL;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("DEC SIXEL Graphics Format");
-  entry->magick_module=ConstantString("SIXEL");
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -1240,18 +1218,15 @@ ModuleExport void UnregisterSIXELImage(void)
 %
 */
 static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
-  Image *image)
+  Image *image,ExceptionInfo *exception)
 {
-  ExceptionInfo
-    *exception;
-
   MagickBooleanType
     status;
 
-  const IndexPacket
-    *indexes;
+  register const Quantum
+    *q;
 
-  ssize_t
+  register ssize_t
     i,
     x;
 
@@ -1263,7 +1238,7 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
     *output;
 
   unsigned char
-    sixel_palette[256 * 3],
+    sixel_palette[256*3],
     *sixel_pixels;
 
   /*
@@ -1273,19 +1248,18 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (IsEventLogging() != MagickFalse)
+  if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  exception=(&image->exception);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-    (void) TransformImageColorspace(image,sRGBColorspace);
+    (void) TransformImageColorspace(image,sRGBColorspace,exception);
   opacity=(-1);
-  if (image->matte == MagickFalse)
+  if (image->alpha_trait == UndefinedPixelTrait)
     {
       if ((image->storage_class == DirectClass) || (image->colors > 256))
-        (void) SetImageType(image,PaletteType);
+        (void) SetImageType(image,PaletteType,exception);
     }
   else
     {
@@ -1297,34 +1271,34 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
         Identify transparent colormap index.
       */
       if ((image->storage_class == DirectClass) || (image->colors > 256))
-        (void) SetImageType(image,PaletteBilevelMatteType);
+        (void) SetImageType(image,PaletteBilevelAlphaType,exception);
       for (i=0; i < (ssize_t) image->colors; i++)
-        if (image->colormap[i].opacity != OpaqueOpacity)
+        if (image->colormap[i].alpha != OpaqueAlpha)
           {
             if (opacity < 0)
               {
                 opacity=i;
                 continue;
               }
-            alpha=(MagickRealType) image->colormap[i].opacity;
-            beta=(MagickRealType) image->colormap[opacity].opacity;
-            if (alpha > beta)
+            alpha=image->colormap[i].alpha;
+            beta=image->colormap[opacity].alpha;
+            if (alpha < beta)
               opacity=i;
           }
       if (opacity == -1)
         {
-          (void) SetImageType(image,PaletteBilevelMatteType);
+          (void) SetImageType(image,PaletteBilevelAlphaType,exception);
           for (i=0; i < (ssize_t) image->colors; i++)
-            if (image->colormap[i].opacity != OpaqueOpacity)
+            if (image->colormap[i].alpha != OpaqueAlpha)
               {
                 if (opacity < 0)
                   {
                     opacity=i;
                     continue;
                   }
-                alpha=(MagickRealType) image->colormap[i].opacity;
-                beta=(MagickRealType) image->colormap[opacity].opacity;
-                if (alpha > beta)
+                alpha=image->colormap[i].alpha;
+                beta=image->colormap[opacity].alpha;
+                if (alpha < beta)
                   opacity=i;
               }
         }
@@ -1340,9 +1314,9 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
   */
   for (i=0; i < (ssize_t) image->colors; i++)
   {
-    sixel_palette[i * 3 + 0] = ScaleQuantumToChar(image->colormap[i].red);
-    sixel_palette[i * 3 + 1] = ScaleQuantumToChar(image->colormap[i].green);
-    sixel_palette[i * 3 + 2] = ScaleQuantumToChar(image->colormap[i].blue);
+    sixel_palette[3*i+0]=ScaleQuantumToChar(image->colormap[i].red);
+    sixel_palette[3*i+1]=ScaleQuantumToChar(image->colormap[i].green);
+    sixel_palette[3*i+2]=ScaleQuantumToChar(image->colormap[i].blue);
   }
 
   /*
@@ -1351,7 +1325,7 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
   output = sixel_output_create(image);
   if (output == (sixel_output_t *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  sixel_pixels =(unsigned char *) AcquireQuantumMemory(image->columns,
+  sixel_pixels=(unsigned char *) AcquireQuantumMemory(image->columns,
     image->rows*sizeof(*sixel_pixels));
   if (sixel_pixels == (unsigned char *) NULL)
     {
@@ -1360,22 +1334,19 @@ static MagickBooleanType WriteSIXELImage(const ImageInfo *image_info,
     }
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    const PixelPacket
-      *p;
-
-    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
-    if (p == (PixelPacket *) NULL)
+    q=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
       break;
-    indexes=GetVirtualIndexQueue(image);
     for (x=0; x < (ssize_t) image->columns; x++)
-      sixel_pixels[y*image->columns+x]=(unsigned char)
-        ((ssize_t) GetPixelIndex(indexes+x));
+    {
+      sixel_pixels[y*image->columns+x]= ((ssize_t) GetPixelIndex(image,q));
+      q+=GetPixelChannels(image);
+    }
   }
-  status = sixel_encode_impl(sixel_pixels, image->columns, image->rows,
-    sixel_palette, image->colors, -1, output);
-  sixel_pixels =(unsigned char *) RelinquishMagickMemory(sixel_pixels);
-  output = (sixel_output_t *) RelinquishMagickMemory(output);
-  if (CloseBlob(image) == MagickFalse)
-    status=MagickFalse;
+  status = sixel_encode_impl(sixel_pixels,image->columns,image->rows,
+    sixel_palette,image->colors,-1,output);
+  sixel_pixels=(unsigned char *) RelinquishMagickMemory(sixel_pixels);
+  output=(sixel_output_t *) RelinquishMagickMemory(output);
+  (void) CloseBlob(image);
   return(status);
 }
